import pygame, sys, random, math
from pygame.locals import *
from pygame.color import *
import pymunk
from pymunk import Vec2d
X, Y = 0, 1

#HEY HEY HEY, STAY OUTTA' MY SOURCE CODE.

# set up pygame
pygame.mixer.init()
pygame.mixer.pre_init(44100, -16, 2, 2048)
pygame.init()

#Small hack
def flip(y):
    return -y+600

#-------------------------
# START CONFIG
#-------------------------
# set up the window #Width, Height, Advanced, Color Depth
screen = pygame.display.set_mode((0, 0),  pygame.FULLSCREEN , 32)
WIDTH = screen.get_width()
HEIGHT = screen.get_height()

#COLOURS
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
#COLOURS
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)

#Center Screen
CENTER = 75

#Jump Speed
JUMP_SPEED = 10

#GRAVITY
GRAVITY = -900

#Coefficent of friction against the surface
FRICTION = 0.5

#Frames Per Second
FPS = 50

#Title
pygame.display.set_caption('Chaos Simulator 5000')

class Player:
	def __init__(self, image, x, y, space):
		self.image = image
		self.pos = image.get_rect().move(x, y)
		self.speed = 0
		self.speedy = 0
		self.jump = False
		self.airtime = 0
		self.direction = "Left"
		self.space = space
		
		angle = 90
		vs = [(-23,26), (23,26), (0,-26)]
		mass = 25 #Yes, twilight is a light-weight....
		moment = pymunk.moment_for_poly(mass, vs)
		self.body = pymunk.Body(mass, moment)
		shape = pymunk.Poly(self.body, vs)
		self.body.apply_impulse(pymunk.Vec2d(60, 60), (0, 0))
		shape.friction = 0.5
		self.body.position = int(WIDTH/2), int(HEIGHT-CENTER)
		self.body.angle = 90   
		space.add(self.body, shape)
		
		self.shape = shape
	
	def move(self):
		if self.speed > 0 and self.direction == "Left":
			self.direction = "Right"
			self.image = pygame.transform.flip(self.image, True, False)
		if self.speed < 0 and self.direction == "Right":
			self.direction = "Left"
			self.image = pygame.transform.flip(self.image, True, False)
		if self.pos.left > WIDTH:
			self.pos.right = 0
		if self.pos.right < 0:
			self.pos.left = WIDTH
		dist = 0
		if self.jump == True or (self.pos.centery) < (HEIGHT - CENTER):
			self.airtime += 0.05
			if self.speedy < JUMP_SPEED:
				self.speedy += 1
			dist = int(((1/2) * 9.81 * (self.airtime**2)) - self.speedy)
		else:
			self.airtime = 0
			self.speedy = 0
		if (self.pos.centery + dist) > (HEIGHT - CENTER):
			dist = HEIGHT - self.pos.centery - CENTER
		self.pos = self.pos.move(self.speed, dist)
	def position_flipped(self):
		return self.shape.body.position


#Game Menu
# 0 -> Main Title
# 1 -> Game Mode
GAMEMENU = {'mode' : 0, 'init' : 0}

#Menus/Gameplay
def main():
	pygame.init()
	clock = pygame.time.Clock()
   
	#Theme music
	pygame.mixer.music.load("Music/Menu/loop.mid")
	pygame.mixer.music.play(-1, 0)
	
	#Load up the background
	backgroundImage = pygame.image.load('Art/Backgrounds/Main_Background.png').convert_alpha()
	backgroundImageS = pygame.transform.scale(backgroundImage, (WIDTH, HEIGHT))
	#Text of doom
	chaosText = pygame.image.load('Art/Text/Chaos_Cloud.png').convert_alpha()
	chaosRec = chaosText.get_rect()
	chaosTextS = pygame.transform.smoothscale(chaosText, (int(chaosRec.right * 2), int(chaosRec.bottom * 2)))

	#Text of doom 2
	simText = pygame.image.load('Art/Text/Simulator_5000.png').convert_alpha()
	simRec = simText.get_rect()
	simTextS = pygame.transform.smoothscale(simText, (int(simRec.right * 2), int(simRec.bottom * 2)))

	#Cloud of doom
	cloud = pygame.image.load('Art/Objects/Chocolate_Rain_Cloud.png').convert_alpha()
	cloudRec = cloud.get_rect()
	cloudS = pygame.transform.smoothscale(cloud, (int(chaosRec.width/1.5), int(chaosRec.height * 2)))
	cloudMove = 0
	cloudMove2 = 0
	cloudMove3 = 0
	cloudHeight2 = random.randint(1, HEIGHT-20)	
	cloudHeight3 = random.randint(1, HEIGHT-20)	

	#Start Button
	startButton = pygame.image.load('Art/Objects/Start_Button.png').convert_alpha()
	startButtonRec = startButton.get_rect()
	startButtonRec = startButtonRec.move(-40,int(HEIGHT-50))	
	
	### Physics stuff
	space = pymunk.Space()
	space.gravity = Vec2d(0.0, -900.0)
	
	#-------------------------------
	# Main menu game loop for pygame
	#------------------------------- 
	while GAMEMENU['mode'] == 0:
		# check for events
		for event in pygame.event.get():
			#Default
			if event.type == QUIT:
				pygame.quit()
				sys.exit()
			#Key down event
			if event.type == KEYDOWN:
				#Exit Key = Esc
				if event.key == K_ESCAPE:
					pygame.quit()
					sys.exit()
			if event.type == MOUSEBUTTONUP:
				if startButtonRec.collidepoint(event.pos[0], event.pos[1]):
					GAMEMENU['mode'] = 1
					GAMEMENU['init'] = 0
		#--------------------
		# DONE EVENT LOOP
		#--------------------
		
		#Move the cloud
		if cloudMove <(int((WIDTH/2) - chaosRec.right + 200)):
			cloudMove += 2
		cloudMove2 += 1
		if cloudMove2 > WIDTH:
			cloudMove2 = 0
			cloudHeight2 = random.randint(1, HEIGHT-20)		
		cloudMove3 += 3
		if cloudMove3 > WIDTH:
			cloudMove3 = 0
			cloudHeight3 = random.randint(1, HEIGHT-20)
				
		# Draw what we have onto the surface
		screen.blit(backgroundImageS, (0,0))
		screen.blit(cloudS,(cloudMove,200))
		screen.blit(cloudS,(cloudMove2-50,cloudHeight2))
		screen.blit(cloudS,(cloudMove3-50,cloudHeight3))
		screen.blit(chaosTextS, (int((WIDTH/2) - chaosRec.right),200))
		screen.blit(simTextS, (int((WIDTH/2) - simRec.right),300))
		screen.blit(startButton, (-40, int(HEIGHT-50)))	
		
		### Flip screen
		pygame.display.flip()
		clock.tick(FPS)
		pygame.display.set_caption("fps: " + str(clock.get_fps()))
		
	#----------------------
	# Setup for Game Mode 1
	#----------------------
	if GAMEMENU['mode'] == 1 and GAMEMENU['init'] == 0:
		pygame.mixer.music.stop()
		
		sounds = []
		pygame.mixer.music.load("Music/Menu/loop2.mid")
		pygame.mixer.music.play(-1, 0)
		sounds.append(pygame.mixer.Sound('Voices/Chaos_Start.ogg'))
		for sound in sounds:
			sound.play()
		
		#Poor Twilight
		twilight = pygame.image.load('Art/Characters/Twilight/twilight_sparkle.png').convert_alpha()
		player = Player(twilight,int(WIDTH/2), flip(int(HEIGHT-CENTER)), space)
		
		# Tossables
		parasprite = pygame.image.load('Art/Objects/Tossable/Parasprite.png').convert_alpha()
		paraspriteRec = parasprite.get_rect()
		parasprite = pygame.transform.scale(parasprite, (int(paraspriteRec.width/15), int(paraspriteRec.height/15)))
		tossables = []
	   
		# Static line
		static_body = pymunk.Body()
		static_lines = [pymunk.Segment(static_body, (0, flip(HEIGHT)), (WIDTH, flip(HEIGHT)), 1.0),
						pymunk.Segment(static_body, (0, 0), (0, flip(HEIGHT)), 1.0),
						pymunk.Segment(static_body, (WIDTH, flip(HEIGHT)), (WIDTH, 0), 1.0),
						]
		for l in static_lines:
			l.friction = 0.5
		space.add(static_lines)
		GAMEMENU['init'] = 1
		
	#----------------------
	# Loop for Game Mode 1
	#----------------------	
	ticks_to_next_spawn = 0
	while GAMEMENU['mode'] == 1:	
		# check for events
		for event in pygame.event.get():
			#Default
			if event.type == QUIT:
				pygame.quit()
				sys.exit()
			#Key down event
			if event.type == KEYDOWN:
				#Exit Key = Esc
				if event.key == K_ESCAPE:
					pygame.quit()
					sys.exit()
				if event.key == K_LEFT or event.key == ord('a'):
					player.body.apply_impulse(pymunk.Vec2d(200, 0), player.body.position)
					player.speed = -3
				if event.key == K_RIGHT or event.key == ord('d'):
					player.body.apply_impulse(pymunk.Vec2d(200, 0), player.body.position)
					player.speed = 3			
				if event.key == K_UP or event.key == ord('w'):
					player.body.apply_impulse(pymunk.Vec2d(0, 300), player.body.position)
					player.jump = True
			if event.type == KEYUP:
				if event.key == K_LEFT or event.key == ord('a'):
					player.speed = 0
				if event.key == K_RIGHT or event.key == ord('d'):
					player.speed = 0
				if event.key == K_UP or event.key == ord('w'):
					player.jump = False
		#--------------------
		# DONE EVENT LOOP
		#--------------------
		#Move the player
		player.move()
		
		ticks_to_next_spawn += 1
		if ticks_to_next_spawn >= 400:
			x = random.randint(20,WIDTH)
			y = 500
			angle = random.random() * math.pi
			vs = [(-23,26), (23,26), (0,-26)]
			mass = 10
			moment = pymunk.moment_for_poly(mass, vs)
			body = pymunk.Body(mass, moment)
			shape = pymunk.Poly(body, vs)
			shape.friction = 0.5
			body.position = x, y
			body.angle = angle
           
			space.add(body, shape)
			tossables.append(shape)
       
		### Update physics
		dt = 1.0/60.0
		for x in range(1):
			space.step(dt)
           
		### Draw stuff
		screen.fill(THECOLORS["white"])
		
		#Draw our surfaces for player
		#screen.blit(player.image, (player.pos.left, player.pos.centery))
		
		#Tossable objects
		for toss in tossables:
			# image draw
			p = toss.body.position
			p = Vec2d(p.x, flip(p.y))
           
			# we need to rotate 180 degrees because of the y coordinate flip
			angle_degrees = math.degrees(toss.body.angle) + 180
			rot = pygame.transform.rotate(parasprite, angle_degrees)
           
			offset = Vec2d(rot.get_size()) / 2.
			p = p - offset
           
			screen.blit(rot, p)
           
			# debug draw
			ps = toss.get_points()
			ps = [(p.x, flip(p.y)) for p in ps]
			ps += [ps[0]]
			pygame.draw.lines(screen, THECOLORS["red"], False, ps, 1)		
			
		# player
		p = player.position_flipped()
		p = Vec2d(p.x, flip(p.y))
           
		# we need to rotate 180 degrees because of the y coordinate flip
		angle_degrees = math.degrees(player.shape.body.angle) + 180
		rot = pygame.transform.rotate(player.image, angle_degrees)
           
		offset = Vec2d(rot.get_size()) / 2.
		p = p - offset
          
		screen.blit(rot, p)
           
		# debug draw
		ps = player.shape.get_points()
		ps = [(p.x, flip(p.y)) for p in ps]
		ps += [ps[0]]
		pygame.draw.lines(screen, THECOLORS["red"], False, ps, 1)	
           
		#Boundaries
		for line in static_lines:
			body = line.body
           
			pv1 = body.position + line.a.rotated(body.angle)
			pv2 = body.position + line.b.rotated(body.angle)
			p1 = pv1.x, flip(pv1.y)
			p2 = pv2.x, flip(pv2.y)
			pygame.draw.lines(screen, THECOLORS["lightgray"], False, [p1,p2], 2)

		### Flip screen
		pygame.display.flip()
		clock.tick(FPS)
		pygame.display.set_caption("fps: " + str(clock.get_fps()))
       
if __name__ == '__main__':
    main()

